import { createStore, createAction } from "../";
// import { Imdux } from "../types";

describe("imdux", () => {
    type State = typeof initalState;
    type Reducers = typeof reducers;

    const initalState = {
        name: "",
        count: 1,
        show: false
    }

    const reducers = {
        increase(draft: State, payload: { inc: number }) {
            draft.count += payload.inc;
        },
        decrease(draft: State, payload: number) {
            draft.count -= payload;
        },
        replace(draft: State, payload: { name: string, count: number, show: boolean }): State {
            return payload;
        },
        show(draft: State, payload: void) {
            draft.show = true;
        },
        hide(draft: State, payload: "1" | "2") {
            draft.show = false;
        }
    }

    it("createStore", () => {
        const home = createAction<State, Reducers>({ initalState, reducers });
        const actions = { home };
        const { Dispatch, Query, redux } = createStore(actions);
        expect(Query.home.name).toBe("");
        expect(Query.home.count).toBe(1);

        let flag = 0;
        redux.subscribe(() => flag = 1)
        Dispatch.home.increase({ inc: 1 });
        expect(Query.home.count).toBe(2);
        expect(flag).toBe(1);

        redux.subscribe(() => flag = 2)
        Dispatch.home.decrease(2);
        expect(Query.home.count).toBe(0);
        expect(flag).toBe(2);

        redux.subscribe(() => flag = 3);
        Dispatch.home.replace({ count: 3, name: "t", show: false });
        expect(Query.home.name).toBe("t");
        expect(Query.home.count).toBe(3);
        expect(flag).toBe(3);

        expect(() => { Query.home = { count: 0, name: "", show: false }; }).toThrowError("modify");
        expect(() => { Query.home.count = 0; }).toThrowError("Cannot assign");
        expect(() => { Query.home.name = ""; }).toThrowError("Cannot assign");
    })

    it("createAction", () => {
        const home = createAction<State, Reducers>({ initalState, reducers });
        const actions = { home };

        expect(() => { home.dispatch.show() }).toThrowError("call");
        expect(() => home.query.name).toThrowError("call");
        expect(() => home.query.count).toThrowError("call");

        const { redux } = createStore(actions);
        expect(home.query.name).toBe("");
        expect(home.query.count).toBe(1);

        let flag = 0;
        redux.subscribe(() => flag = 1)
        home.dispatch.increase({ inc: 1 });
        expect(home.query.count).toBe(2);
        expect(flag).toBe(1);

        redux.subscribe(() => flag = 2)
        home.dispatch.decrease(2);
        expect(home.query.count).toBe(0);
        expect(flag).toBe(2);

        redux.subscribe(() => flag = 3);
        home.dispatch.replace({ count: 3, name: "t", show: false });
        expect(home.query.name).toBe("t");
        expect(home.query.count).toBe(3);
        expect(flag).toBe(3);

        redux.subscribe(() => flag = 4);

        home.dispatch.show()
        expect(home.query.show).toBe(true);
        expect(flag).toBe(4);


        expect(() => { home.query = { count: 0, name: "", show: false }; }).toThrowError("modify");
        expect(() => { home.query.count = 0; }).toThrowError("Cannot assign");
        expect(() => { home.query.name = ""; }).toThrowError("Cannot assign");
    })

    it("types", () => {
        const home = createAction<State, Reducers>({ initalState, reducers });
        const actions = { home };
        const { Dispatch } = createStore(actions);

        Dispatch.home.hide("1");
        Dispatch.home.hide("2");
    })
});
